
#ifndef VTKIMAGINGSTATISTICS_EXPORT_H
#define VTKIMAGINGSTATISTICS_EXPORT_H

#ifdef VTKIMAGINGSTATISTICS_STATIC_DEFINE
#  define VTKIMAGINGSTATISTICS_EXPORT
#  define VTKIMAGINGSTATISTICS_NO_EXPORT
#else
#  ifndef VTKIMAGINGSTATISTICS_EXPORT
#    ifdef ImagingStatistics_EXPORTS
        /* We are building this library */
#      define VTKIMAGINGSTATISTICS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKIMAGINGSTATISTICS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKIMAGINGSTATISTICS_NO_EXPORT
#    define VTKIMAGINGSTATISTICS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKIMAGINGSTATISTICS_DEPRECATED
#  define VTKIMAGINGSTATISTICS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKIMAGINGSTATISTICS_DEPRECATED_EXPORT
#  define VTKIMAGINGSTATISTICS_DEPRECATED_EXPORT VTKIMAGINGSTATISTICS_EXPORT VTKIMAGINGSTATISTICS_DEPRECATED
#endif

#ifndef VTKIMAGINGSTATISTICS_DEPRECATED_NO_EXPORT
#  define VTKIMAGINGSTATISTICS_DEPRECATED_NO_EXPORT VTKIMAGINGSTATISTICS_NO_EXPORT VTKIMAGINGSTATISTICS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKIMAGINGSTATISTICS_NO_DEPRECATED
#    define VTKIMAGINGSTATISTICS_NO_DEPRECATED
#  endif
#endif

#endif /* VTKIMAGINGSTATISTICS_EXPORT_H */
